@@ -19,6 +19,7 @@
 #include <libsoup/soup.h>
 #include <libedataserver/e-time-utils.h>
 
+#include "sunone-util.h"
 #include "sunone-connection.h"
 #include "sunone-message.h"
 #include "calendar/cal-backend-wcap.h"
@@ -27,8 +28,7 @@
 #define IS_CONNECTED(connection,return_value) {if (!connection->priv->wcap_session_id) return return_value;}
 #define NEW_URI(uri) {uri = soup_uri_new (connection->priv->server_uri); \
 		if (uri && uri->path) {g_free (uri->path); uri->path = NULL;} \
-		if (uri && uri->query) {g_free (uri->query); uri->query = NULL;}}	\
-		if (uri) uri->broken_encoding = TRUE;
+		if (uri && uri->query) {g_free (uri->query); uri->query = NULL;}}
 #define IS_CNC_WCAP_2_0(connection) ( !strncmp (connection->priv->wcap_version, "2.0", 3) )
 
 struct _SunOneConnectionPrivate {
@@ -36,6 +36,7 @@ struct _SunOneConnectionPrivate {
 
 	char *server_uri;
 	char *user;
+	char *uid;
 	char *password;
 	char *wcap_session_id;
 	char *wcap_version;
@@ -62,7 +63,6 @@ static guint sunone_connection_storeevents_by_mail (SunOneConnection *connection
 
 static GObjectClass *parent_class = NULL;
 
-
 static char *
 query_encode (const char *part, gboolean escape_unsafe,
 	      const char *escape_extra)
@@ -87,7 +87,7 @@ query_encode (const char *part, gboolean escape_unsafe,
 }
 
 static void
-append_to_query (SoupUri *uri, const char *param, const char *value)
+append_to_query (SoupURI *uri, const char *param, const char *value)
 {
 	char *tmp, *enc;
 	
@@ -103,7 +103,7 @@ append_to_query (SoupUri *uri, const char *param, const char *value)
 }
 
 static void
-append_to_query_int (SoupUri *uri, const char *param, int value)
+append_to_query_int (SoupURI *uri, const char *param, int value)
 {
 	char *tmp;
 	
@@ -135,7 +135,7 @@ get_stringv (const char **stringv)
 
 
 static void
-append_to_query_stringv (SoupUri *uri, const char *param, const char **stringv)
+append_to_query_stringv (SoupURI *uri, const char *param, const char **stringv)
 {
 	char *string;
 	
@@ -150,7 +150,7 @@ append_to_query_stringv (SoupUri *uri, const char *param, const char **stringv)
 }
 
 static void
-append_to_query_text (SoupUri *uri, const char *param, ECalComponentText *text)
+append_to_query_text (SoupURI *uri, const char *param, ECalComponentText *text)
 {
 	if (text->value)
 		append_to_query (uri, param, text->value);
@@ -159,7 +159,7 @@ append_to_query_text (SoupUri *uri, const char *param, ECalComponentText *text)
 }
 
 static void
-append_to_query_text_list (SoupUri *uri, const char *param, GSList *text_list)
+append_to_query_text_list (SoupURI *uri, const char *param, GSList *text_list)
 {
 	GSList *l;
 	char *string;
@@ -184,9 +184,9 @@ append_to_query_text_list (SoupUri *uri, const char *param, GSList *text_list)
 }
 
 static void
-append_to_query_datetime (SoupUri *uri, const char *param, ECalComponentDateTime *dt, gboolean zero)
+append_to_query_datetime (SoupURI *uri, const char *param, ECalComponentDateTime *dt, gboolean zero)
 {
-	const char *string;
+	char *string;
 
 	if (!dt->value || !icaltime_is_valid_time (*dt->value)) {
 		if (zero) 
@@ -196,11 +196,12 @@ append_to_query_datetime (SoupUri *uri, const char *param, ECalComponentDateTime
 	
 	string = icaltime_as_ical_string (*dt->value);
 	append_to_query (uri, param, string);	
+	g_free (string);
 	e_cal_component_free_datetime (dt);
 }
 
 static void
-append_to_query_attendees (SoupUri *uri, const char *param, GSList *attendee_list, SunOneConnection *connection, const char *calid, const char *email)
+append_to_query_attendees (SoupURI *uri, const char *param, GSList *attendee_list, SunOneConnection *connection, const char *calid, const char *email)
 {
 	GSList *l;
 	GString *string;
@@ -300,12 +301,19 @@ append_to_query_attendees (SoupUri *uri, const char *param, GSList *attendee_lis
 			g_string_append_printf (string, "^LANGUAGE=%s", attendee->language);
 
 		/* Email Address */		
-		if (!strncasecmp (attendee->value, "mailto:", 7) && strchr (attendee->value, '@')) {
-			/*if account is in the attendees, change its email to calid*/
-			if(!strcasecmp (email, attendee->value + 7)) {
-				g_string_append_printf (string, "^%s;", calid);
+		if (!strncasecmp (attendee->value, "mailto:", 7)) {
+			if (strchr (attendee->value, '@')) {
+				/* mailto a real email address,
+				 * if account is in the attendees,
+				 * change its email to calid
+				 */
+				if(!strcasecmp (email, attendee->value + 7)) {
+					g_string_append_printf (string, "^%s;", calid);
+				} else
+					g_string_append_printf (string, "^%s;", attendee->value);
 			} else
-				g_string_append_printf (string, "^%s;", attendee->value);
+				/* mailto, but a calid, strip it */
+				g_string_append_printf (string, "^%s;", attendee->value + 7);
 
 		} else
 			g_string_append_printf (string, "^%s;", attendee->value);			
@@ -323,7 +331,7 @@ append_to_query_attendees (SoupUri *uri, const char *param, GSList *attendee_lis
 }
 
 static void
-append_to_query_exdate_list (SoupUri *uri, const char *param, GSList *dt_list)
+append_to_query_exdate_list (SoupURI *uri, const char *param, GSList *dt_list)
 {
 	GSList *l;
 	char *string;
@@ -332,16 +340,19 @@ append_to_query_exdate_list (SoupUri *uri, const char *param, GSList *dt_list)
 	for (l = dt_list; l != NULL; l = l->next) {
 		ECalComponentDateTime *dt = l->data;
 		char *tmp = string;
+		char *tstr;
 		
 		if (!dt->value)
 			continue;
+		tstr = icaltime_as_ical_string (*dt->value);
 		
 		if (string) {
-			string = g_strconcat (string, ";", icaltime_as_ical_string (*dt->value), NULL);
+			string = g_strconcat (string, ";", tstr, NULL);
 			g_free (tmp);
 		} else {
-			string = g_strdup (icaltime_as_ical_string (*dt->value));
+			string = g_strdup (tstr);
 		}
+		g_free (tstr);
 	}
 	if (string)
 		append_to_query (uri, param, string);
@@ -350,7 +361,7 @@ append_to_query_exdate_list (SoupUri *uri, const char *param, GSList *dt_list)
 }
 
 static void
-append_to_query_recur_list (SoupUri *uri, const char *param, GSList *r_list)
+append_to_query_recur_list (SoupURI *uri, const char *param, GSList *r_list)
 {
 	GSList *l;
 	char *string;
@@ -376,7 +387,7 @@ append_to_query_recur_list (SoupUri *uri, const char *param, GSList *r_list)
 }
 
 static void
-append_to_query_classification (SoupUri *uri, const char *param, ECalComponentClassification class)
+append_to_query_classification (SoupURI *uri, const char *param, ECalComponentClassification class)
 {
 	switch (class) {
 	case E_CAL_COMPONENT_CLASS_PUBLIC:
@@ -395,7 +406,7 @@ append_to_query_classification (SoupUri *uri, const char *param, ECalComponentCl
 
 
 static void
-append_to_query_status (SoupUri *uri, const char *param, icalproperty_status status)
+append_to_query_status (SoupURI *uri, const char *param, icalproperty_status status)
 {
 	switch (status) {
 	case ICAL_STATUS_CONFIRMED:
@@ -428,7 +439,7 @@ append_to_query_status (SoupUri *uri, const char *param, icalproperty_status sta
 }
 
 static void
-append_to_query_compstate (SoupUri *uri, const char *param, SunOneCompState state)
+append_to_query_compstate (SoupURI *uri, const char *param, SunOneCompState state)
 {
 	const char *statev[7];
 	int i;
@@ -463,7 +474,7 @@ append_to_query_compstate (SoupUri *uri, const char *param, SunOneCompState stat
 }
 
 static void
-append_to_query_geo (SoupUri *uri, const char *param, struct icalgeotype *geo)
+append_to_query_geo (SoupURI *uri, const char *param, struct icalgeotype *geo)
 {
 	char *tmp;
 	
@@ -476,14 +487,14 @@ append_to_query_geo (SoupUri *uri, const char *param, struct icalgeotype *geo)
 }
 
 static void
-append_to_query_alarms (SoupUri *uri, SunOneConnection *connection, ECalComponent *comp)
+append_to_query_alarms (SoupURI *uri, SunOneConnection *connection, ECalComponent *comp)
 {
 	GList *auids, *l;
 	ECalComponentAlarm *alarm;
 	ECalComponentAlarmAction action;
 	ECalComponentAlarmTrigger trigger;
 	char *atype;
-	const char *dtstring;
+	char *dtstring;
 
 	if (!e_cal_component_has_alarms (comp)) {
 		
@@ -522,7 +533,8 @@ append_to_query_alarms (SoupUri *uri, SunOneConnection *connection, ECalComponen
 		if (trigger.type == E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE)
 			dtstring = icaltime_as_ical_string (trigger.u.abs_time);
 		else
-			dtstring = icaldurationtype_as_ical_string (trigger.u.rel_duration);
+			dtstring =
+				g_strdup (icaldurationtype_as_ical_string (trigger.u.rel_duration));
 		
 		if (atype != NULL && dtstring != NULL) {
 			ECalComponentText text;
@@ -562,6 +574,7 @@ append_to_query_alarms (SoupUri *uri, SunOneConnection *connection, ECalComponen
 				}
 			}
 		}
+		g_free (dtstring);
 		
 		e_cal_component_alarm_free (alarm);
 	}
@@ -615,16 +628,16 @@ sunone_connection_init (SunOneConnection *object)
 
 
 static gboolean
-parse_session_object (SunOneConnection *connection, const char *body)
+parse_session_object (SunOneConnection *connection, SoupMessage *msg)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
 	icalcomponent *icalcomp;
 	icalcomponent_kind kind;
 	icalproperty *icalprop;
 
-	g_return_val_if_fail (body != NULL, FALSE);
+	g_return_val_if_fail (msg != NULL, FALSE);
 
-	icalcomp = icalparser_parse_string (body);
+	icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 	if (!icalcomp)
 		return FALSE;
 
@@ -660,8 +673,9 @@ static guint
 sunone_login (SunOneConnection *connection, gboolean refresh)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *login_uri;
+	SoupURI *login_uri;
 	SoupMessage *msg;
+	icalcomponent *icalcomp;
 	guint error_code = SOUP_STATUS_CANT_CONNECT;
 	char *encode_password = NULL;
 	
@@ -690,7 +704,7 @@ sunone_login (SunOneConnection *connection, gboolean refresh)
 
 	error_code = msg->status_code;
 	if (SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
-		if (parse_session_object (connection, msg->response.body)) {
+		if (parse_session_object (connection, msg)) {
 			if (priv->timeout_id == -1)
 				priv->timeout_id = g_timeout_add (REFRESH_INTERVAL,
 									(GSourceFunc) reconnect_timeout_cb,
@@ -698,9 +712,34 @@ sunone_login (SunOneConnection *connection, gboolean refresh)
 			priv->is_open = TRUE;
 		}
 	}
-
+	/* parse the returned data */
+	icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 	g_object_unref (msg);
 	soup_uri_free (login_uri);
+	if (icalcomp) {
+		icalproperty *icalprop =
+			icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+
+		while (icalprop) {
+			const char *x_name, *x_val;
+
+			x_name = icalproperty_get_x_name (icalprop);
+			x_val = icalproperty_get_x (icalprop);
+			/*
+			 * if user id is different from user name,
+			 * we have to remember user id and compare
+			 * it with primary owner of the folder to decide
+			 * if user is the owner of the folder.
+			 */
+			if (!strcmp (x_name, "X-NSCP-WCAP-USER-ID") &&
+				priv->user && x_val && strcmp (priv->user, x_val)) {
+				priv->uid = g_strdup (x_val);
+				fprintf (stderr, "uid:%s\n", x_val);
+			}
+			icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+		}	
+		icalcomponent_free (icalcomp);
+	}
 
 	return error_code;
 }
@@ -709,7 +748,7 @@ static guint
 sunone_logout (SunOneConnection *connection)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *logout_uri;
+	SoupURI *logout_uri;
 	SoupMessage *msg;
 	guint error_code;
 
@@ -793,6 +832,10 @@ sunone_connection_dispose (GObject *object)
 			g_free (priv->user);
 			priv->user = NULL;
 		}
+		if (priv->uid) {
+			g_free (priv->uid);
+			priv->uid = NULL;
+		}
 		if (priv->password) {
 			g_free (priv->password);
 			priv->password = NULL;
@@ -837,12 +880,13 @@ sunone_connection_finalize (GObject *object)
 }
 
 SunOneConnection *
-sunone_connection_new (const char *server_uri,
+sunone_connection_new (const char *server_uri, const char *proxy_uri,
 		       const char *user,
 		       const char *password)
 {
 	SunOneConnection *connection;
 	SunOneConnectionPrivate *priv;
+	SoupURI *suri;
 
 	connection = g_object_new (SUNONE_CONNECTION_TYPE, NULL);
 	priv = connection->priv;
@@ -850,6 +894,11 @@ sunone_connection_new (const char *server_uri,
 	priv->user = g_strdup (user);
 	priv->password = g_strdup (password);
 	priv->is_open = FALSE;
+	if (proxy_uri && priv->soup_session) {
+		suri = soup_uri_new (proxy_uri);
+		g_object_set (G_OBJECT (priv->soup_session), SOUP_SESSION_PROXY_URI, suri, NULL);
+		soup_uri_free (suri);
+	}
 	
 	return connection;
 }
@@ -996,6 +1045,15 @@ sunone_connection_get_user(SunOneConnection *connection)
 	return (const char *)priv->user;
 }
 
+const char *
+sunone_connection_get_uid(SunOneConnection *connection)
+{
+	SunOneConnectionPrivate *priv = connection->priv;
+
+	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), NULL);
+	return (const char *)priv->uid;
+}
+
 void
 sunone_connection_set_user (SunOneConnection *connection, const char *user)
 {
@@ -1041,7 +1099,7 @@ SunOneConnectionPreferences *
 sunone_connection_get_preferences (SunOneConnection *connection)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *get_uri;
+	SoupURI *get_uri;
 	SoupMessage *msg;
 	guint ret;
 	icalcomponent *icalcomp;
@@ -1080,7 +1138,7 @@ sunone_connection_get_preferences (SunOneConnection *connection)
 	}
 
 	/* parse the returned data */
-	icalcomp = icalparser_parse_string (msg->response.body);
+	icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 	g_object_unref (msg);
 	if (!icalcomp) {
 		g_static_mutex_unlock (&mutex);
@@ -1204,7 +1262,7 @@ guint
 sunone_connection_set_preferences (SunOneConnection *connection, SunOneConnectionPreferences *prefs)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *set_uri;
+	SoupURI *set_uri;
 	SoupMessage *msg;
 	gchar *s, *s1 = NULL;
 	guint retval;
@@ -1332,7 +1390,7 @@ guint
 sunone_connection_createcalendar (SunOneConnection *connection, const gchar *calid, gchar **real_calid)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *suri;
+	SoupURI *suri;
 	SoupMessage *msg;
 	guint retval;
 	GList *acls = NULL;
@@ -1373,7 +1431,7 @@ sunone_connection_createcalendar (SunOneConnection *connection, const gchar *cal
 		icalcomponent *icalcomp;
 
 		/* parse the response to get the calid as generated in the server */
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (icalcomp) {
 			icalproperty *icalprop;
 
@@ -1411,7 +1469,7 @@ sunone_connection_createcalendar (SunOneConnection *connection, const gchar *cal
 			SunOneCalendarSubscription *sub;
 
 			sub = g_new0 (SunOneCalendarSubscription, 1);
-			sub->calid = g_strdup_printf (calidtmp);
+			sub->calid = g_strdup (calidtmp);
 			sub->display_name = g_strdup (calid);
 
 			prefs->subscriptions = g_list_append (prefs->subscriptions, sub);
@@ -1438,7 +1496,7 @@ guint
 sunone_connection_deletecalendar (SunOneConnection *connection, const gchar *calid)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *suri;
+	SoupURI *suri;
 	SoupMessage *msg;
 	guint retval;
 	SunOneConnectionPreferences *prefs;
@@ -1586,7 +1644,7 @@ sunone_connection_get_calprops (SunOneConnection *connection, const gchar *calid
 	SunOneConnectionPrivate *priv = connection->priv;
 	gchar *calid_hash = NULL;
 	SunOneCalendarProperties *calprops = NULL;
-	SoupUri *get_uri;
+	SoupURI *get_uri;
 	SoupMessage *msg;
 	icalcomponent *icalcomp;
 	SunOneCalendarProperties *clone;
@@ -1621,7 +1679,7 @@ sunone_connection_get_calprops (SunOneConnection *connection, const gchar *calid
 		}
 
 		/* parse the returned data */
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		g_object_unref (msg);
 		if (!icalcomp) {
 			g_static_mutex_unlock (&mutex);
@@ -1709,7 +1767,7 @@ sunone_connection_set_calprops (SunOneConnection *connection,
 				SunOneCalendarProperties *calprops)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *set_uri;
+	SoupURI *set_uri;
 	SoupMessage *msg;
 	guint retval;
 	gchar *s;
@@ -1767,7 +1825,7 @@ GList *
 sunone_connection_list_calids (SunOneConnection *connection)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *get_uri;
+	SoupURI *get_uri;
 	SoupMessage *msg;
 	icalcomponent *icalcomp, *subcomp;
 	SunOneCalendarProperties *calprops;
@@ -1790,7 +1848,7 @@ sunone_connection_list_calids (SunOneConnection *connection)
 	soup_uri_free (get_uri);
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (icalcomp) {
 			subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
 			while (subcomp) {
@@ -1816,7 +1874,7 @@ GList *
 sunone_connection_search_calids (SunOneConnection *connection, SunOneSearchField field, SunOneSearchBy by, const char *search_str)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *get_uri;
+	SoupURI *get_uri;
 	SoupMessage *msg;
 	icalcomponent *icalcomp, *subcomp;
 	SunOneCalendarProperties *calprops;
@@ -1853,20 +1911,38 @@ sunone_connection_search_calids (SunOneConnection *connection, SunOneSearchField
 	soup_uri_free (get_uri);
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
-		if (icalcomp) {
-			subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
-			while (subcomp) {
-				calprops = calprops_from_icalcomp (subcomp);
-				if (calprops)
-					calids = g_list_append (calids, calprops);
+		char *tmp_str;
+		char *vcal_str;
 
-				subcomp = icalcomponent_get_next_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
+		tmp_str = g_strndup (msg->response_body->data, msg->response_body->length);
+		if (tmp_str) {
+			vcal_str = g_strdup_printf ("BEGIN:VCALENDAR\n%s\nEND:VCALENDAR\n", tmp_str);
+			g_free (tmp_str);
+			if (vcal_str) {
+				icalcomp = sunone_util_icalparser_parse_string (vcal_str, strlen (vcal_str));
+				g_free (vcal_str);
+				if (icalcomp) {
+					subcomp = icalcomponent_get_first_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
+					while (subcomp) {
+						/* Because we wrap the result in a vcalender,
+						 * we always get at least one sub component in icalcompoent.
+						 * So we use ICAL_X_PROPERTY to identify if this sub component
+						 * is the only one in search result or it is just fake.
+						 */
+						if (icalcomponent_get_first_property (subcomp, ICAL_X_PROPERTY) != NULL) {
+							calprops = calprops_from_icalcomp (subcomp);
+							if (calprops)
+								calids = g_list_append (calids, calprops);
+						}
+
+						subcomp = icalcomponent_get_next_component (icalcomp, ICAL_VCALENDAR_COMPONENT);
+					}
+
+					icalcomponent_free (icalcomp);
+				} else
+					g_warning ("Error: could not parse response's body");
 			}
-
-			icalcomponent_free (icalcomp);
-		} else
-			g_warning ("Error: could not parse response's body");
+		}
 	} else
 		g_warning ("Error: %s", msg->reason_phrase);
 
@@ -1899,7 +1975,7 @@ sunone_connection_get_all_timezones (SunOneConnection *connection,
 				     icalcomponent **icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 
@@ -1920,7 +1996,7 @@ sunone_connection_get_all_timezones (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -1940,7 +2016,7 @@ sunone_connection_deletetodos_by_id (SunOneConnection *connection,
 				     SunOneModType mod)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 
@@ -1976,7 +2052,7 @@ sunone_connection_deleteevents_by_id (SunOneConnection *connection,
 				      SunOneModType mod)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 
@@ -2051,9 +2127,11 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 					      time_t *server_time)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2062,14 +2140,23 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/fetchcomponents_by_lastmod.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&component-type=%s&relativealarm=4&fmt-out=text/calendar&compressed=1",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0",
+		tsstr, testr,
 		type_to_string (type));
+	g_free (tsstr);
+	g_free (testr);
 
 	append_to_query_compstate (fetch_uri, "compstate", state);
 	
@@ -2079,14 +2166,14 @@ sunone_connection_fetchcomponents_by_lastmod (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 
 		if (server_time) {
 			const char *date;
 			
-			date = soup_message_get_header (msg->response_headers, "Date");
+			date = soup_message_headers_get (msg->response_headers, "Date");
 			*server_time = parse_http_date (date);
 			if (*server_time < 0)
 				*server_time = 0;
@@ -2108,9 +2195,11 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 				icalcomponent **icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2119,14 +2208,23 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/fetch_deletedcomponents.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&component-type=%s&fmt-out=text/calendar&recurring=0",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0",
+		tsstr, testr,
 		type_to_string (type));
+	g_free (tsstr);
+	g_free (testr);
 
 	
 	/* send the request to the server */
@@ -2135,7 +2233,7 @@ sunone_connection_fetch_deletedcomponents (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2154,22 +2252,33 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 				icalcomponent **icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
+	char *tsstr;
+	char *testr;
 
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
 
 	/* prepare the URI */
 	NEW_URI (fetch_uri);
+	if (!icaltime_is_null_time (dtstart))
+		tsstr = icaltime_as_ical_string (dtstart);
+	else
+		tsstr = g_strdup ("0");
+	if (!icaltime_is_null_time (dtend))
+		testr = icaltime_as_ical_string (dtend);
+	else
+		testr = g_strdup ("0");
 	fetch_uri->path = g_strdup ("/get_freebusy.wcap");
 	fetch_uri->query = g_strdup_printf (
 		"id=%s&calid=%s&dtstart=%s&dtend=%s&tzid=UTC&tzidout=UTC&busyonly=1&fmt-out=text/calendar&compressed=1",
 		priv->wcap_session_id,
 		calid,
-		!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-		!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0");
+		tsstr, testr);
+	g_free (tsstr);
+	g_free (testr);
 
 	/* send the request to the server */
 	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
@@ -2177,7 +2286,7 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2187,16 +2296,27 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 
 	/* For WCAP 3.0, maybe calid is a mail address, try again */
 	if (!IS_CNC_WCAP_2_0(connection) && !SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
+		char *tsstr;
+		char *testr;
 
 		/* prepare the URI */
 		NEW_URI (fetch_uri);
+		if (!icaltime_is_null_time (dtstart))
+			tsstr = icaltime_as_ical_string (dtstart);
+		else
+			tsstr = g_strdup ("0");
+		if (!icaltime_is_null_time (dtend))
+			testr = icaltime_as_ical_string (dtend);
+		else
+			testr = g_strdup ("0");
 		fetch_uri->path = g_strdup ("/get_freebusy.wcap");
 		fetch_uri->query = g_strdup_printf (
 			"id=%s&mail=%s&dtstart=%s&dtend=%s&tzid=UTC&tzidout=UTC&busyonly=1&fmt-out=text/calendar&compressed=1",
 			priv->wcap_session_id,
 			calid,
-			!icaltime_is_null_time (dtstart) ? icaltime_as_ical_string (dtstart) : "0",
-			!icaltime_is_null_time (dtend) ? icaltime_as_ical_string (dtend) : "0");
+			tsstr, testr);
+		g_free (tsstr);
+		g_free (testr);
 
 		/* send the request to the server */
 		msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
@@ -2204,7 +2324,7 @@ sunone_connection_get_freebusy (SunOneConnection *connection,
 		retval = msg->status_code;
 
 		if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-			*icalcomp = icalparser_parse_string (msg->response.body);
+			*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 			if (!*icalcomp)
 				retval = SOUP_STATUS_MALFORMED;
 		}
@@ -2221,7 +2341,7 @@ guint
 sunone_connection_import (SunOneConnection *connection, const char *calid, icalcomponent *icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *uri;
+	SoupURI *uri;
 	SoupMessage *msg;
 	guint retval;
 	char *ical_string, *import_msg, *content_type, *err;
@@ -2248,7 +2368,7 @@ sunone_connection_import (SunOneConnection *connection, const char *calid, icalc
 	import_msg = g_strdup_printf ("-----------------------------33111928916708\r\nContent-Disposition: form-data; name=\"Upload\"; filename=\"ical1.ics\"\r\n\r\n%s-----------------------------33111928916708--", ical_string);
 	
 	content_type = "multipart/form-data;boundary=---------------------------33111928916708";	
-	msg = sunone_message_new_full_from_uri (uri, SOUP_METHOD_POST, content_type, SOUP_BUFFER_SYSTEM_OWNED,
+	msg = sunone_message_new_full_from_uri (uri, SOUP_METHOD_POST, content_type, SOUP_MEMORY_TAKE,
 						import_msg, strlen (import_msg));
 
 	/* send the request to the server */
@@ -2258,13 +2378,13 @@ sunone_connection_import (SunOneConnection *connection, const char *calid, icalc
 	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) 
 		retval = msg->status_code;
 
-	if (msg->response.body) {
+	if (msg->response_body->data) {
 		if (IS_CNC_WCAP_2_0 (connection)) {
-			err = strstr (msg->response.body, "var errno=");
+			err = strstr (msg->response_body->data, "var errno=");
 			err += 10;
 		}
 		else {
-			err = strstr (msg->response.body, "X-NSCP-WCAP-ERRNO:");
+			err = strstr (msg->response_body->data, "X-NSCP-WCAP-ERRNO:");
 			err += 18;
 		}
 
@@ -2310,7 +2430,7 @@ sunone_connection_storeevents_by_mail (SunOneConnection *connection, const char
 {
 	SunOneConnectionPrivate *priv = connection->priv;
 	ECalComponent *comp;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 	icalcomponent *real_icalcomp;
@@ -2504,7 +2624,7 @@ sunone_connection_storeevents_by_mail (SunOneConnection *connection, const char
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code) && return_icalcomp) {
-		*return_icalcomp = icalparser_parse_string (msg->response.body);
+		*return_icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*return_icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2546,7 +2666,7 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 {
 	SunOneConnectionPrivate *priv = connection->priv;
 	ECalComponent *comp;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 	icalcomponent *real_icalcomp;
@@ -2562,6 +2682,7 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 	icalproperty_status status;
 	const char *const_string;
 	int *num;
+	char *tstr;
 	
 	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
 	g_return_val_if_fail (calid != NULL, SUNONE_ERROR_ILLEGAL_CALID_NAME);
@@ -2601,7 +2722,12 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 
 	/* Completed */
 	e_cal_component_get_completed (comp, &t);
-	append_to_query (fetch_uri, "completed", t ? icaltime_as_ical_string (*t) : "0");
+	if (t)
+		tstr = icaltime_as_ical_string (*t);
+	else
+		tstr = g_strdup ("0");
+	append_to_query (fetch_uri, "completed", tstr);
+	g_free (tstr);
 	
 	/* Description */
 	e_cal_component_get_description_list (comp, &slist);
@@ -2667,6 +2793,12 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 	if (num) {
 		append_to_query_int (fetch_uri, "percent", *num);
 		e_cal_component_free_percent (num);
+	} else {
+		/*When one marks a completed task as uncompleted, evolution will remove
+		 *percent property, this would cause problem on some calendar servers,
+		 *so we need to set percent property to 0
+		 */
+		append_to_query_int (fetch_uri, "percent", 0);
 	}
 	
 	/* Priority */
@@ -2734,7 +2866,7 @@ sunone_connection_storetodos (SunOneConnection *connection, const char *calid, i
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code) && return_icalcomp) {
-		*return_icalcomp = icalparser_parse_string (msg->response.body);
+		*return_icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*return_icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2756,7 +2888,7 @@ sunone_connection_verifyevents_by_ids (SunOneConnection *connection,
 				       icalcomponent **icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	char *post_string, *content_type;
 	guint retval;
@@ -2782,14 +2914,14 @@ sunone_connection_verifyevents_by_ids (SunOneConnection *connection,
 	
 	content_type = "application/x-www-form-urlencoded";
 	/* send the request to the server */
-	msg = sunone_message_new_full_from_uri (fetch_uri, SOUP_METHOD_POST, content_type, SOUP_BUFFER_SYSTEM_OWNED,
+	msg = sunone_message_new_full_from_uri (fetch_uri, SOUP_METHOD_POST, content_type, SOUP_MEMORY_TAKE,
 		post_string, strlen (post_string));
 
 	sunone_message_send (connection, msg);
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2809,7 +2941,7 @@ sunone_connection_verifytodos_by_ids (SunOneConnection *connection,
 				      icalcomponent **icalcomp)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	char *post_string, *content_type;
 	guint retval;
@@ -2834,14 +2966,14 @@ sunone_connection_verifytodos_by_ids (SunOneConnection *connection,
 	
 	content_type = "application/x-www-form-urlencoded";
 	/* send the request to the server */
-	msg = sunone_message_new_full_from_uri (fetch_uri, SOUP_METHOD_POST, content_type, SOUP_BUFFER_SYSTEM_OWNED,
+	msg = sunone_message_new_full_from_uri (fetch_uri, SOUP_METHOD_POST, content_type, SOUP_MEMORY_TAKE,
 		post_string, strlen (post_string));
 
 	sunone_message_send (connection, msg);
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
+		*icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!*icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 	}
@@ -2852,90 +2984,6 @@ sunone_connection_verifytodos_by_ids (SunOneConnection *connection,
 	return retval;
 }
 
-#if 0
-guint
-sunone_connection_fetchevents_by_id (SunOneConnection *connection,
-				      const char *calid,
-				      const char *uid,
-//				      const char **ridv,
-					  SunOneModType mod,
-				      icalcomponent **icalcomp)
-{
-	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
-	SoupMessage *msg;
-	guint retval;
-
-	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
-
-	IS_CONNECTED (connection, SOUP_STATUS_CANT_CONNECT);
-	
-	/* prepare the URI */
-	NEW_URI (fetch_uri);
-	fetch_uri->path = g_strdup ("/fetchevents_by_id.wcap");
-	fetch_uri->query = g_strdup_printf (
-		"id=%s&calid=%s&fmt-out=text/calendar&uid=%s&mod=%d&compressed=1",
-		priv->wcap_session_id,
-		calid, uid, mod);
-
-	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
-	sunone_message_send (connection, msg);
-	retval = msg->status_code;
-
-	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
-		if (!*icalcomp)
-			retval = SOUP_STATUS_MALFORMED;
-	}
-
-	g_object_unref (msg);
-	soup_uri_free (fetch_uri);
-
-	return retval;
-}
-
-guint
-sunone_connection_fetchtodos_by_id (SunOneConnection *connection,
-				      const char *calid,
-				      const char *uid,
-//				      const char **ridv,
-					  SunOneModType mod,
-				      icalcomponent **icalcomp)
-{
-	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
-	SoupMessage *msg;
-	guint retval;
-
-	g_return_val_if_fail (IS_SUNONE_CONNECTION (connection), SOUP_STATUS_CANCELLED);
-
-	IS_CONNECTED (connection, SOUP_STATUS_CANT_CONNECT);
-	
-	/* prepare the URI */
-	NEW_URI (fetch_uri);
-	fetch_uri->path = g_strdup ("/fetchtodos_by_id.wcap");
-	fetch_uri->query = g_strdup_printf (
-		"id=%s&calid=%s&fmt-out=text/calendar&uid=%s&mod=%d",
-		priv->wcap_session_id,
-		calid, uid, mod);
-
-	msg = sunone_message_new_from_uri (fetch_uri, SOUP_METHOD_GET);
-	sunone_message_send (connection, msg);
-	retval = msg->status_code;
-
-	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		*icalcomp = icalparser_parse_string (msg->response.body);
-		if (!*icalcomp)
-			retval = SOUP_STATUS_MALFORMED;
-	}
-
-	g_object_unref (msg);
-	soup_uri_free (fetch_uri);
-
-	return retval;
-}
-#endif
-	
 void
 sunone_connection_add_wcap (SunOneConnection *connection, const char *key, gpointer wcap)
 {
@@ -2960,7 +3008,7 @@ guint
 sunone_connection_version (SunOneConnection *connection)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupUri *fetch_uri;
+	SoupURI *fetch_uri;
 	SoupMessage *msg;
 	guint retval;
 	icalcomponent *icalcomp;
@@ -2982,7 +3030,7 @@ sunone_connection_version (SunOneConnection *connection)
 	retval = msg->status_code;
 
 	if (SUNONE_ERROR_IS_SUCCESSFUL (msg->status_code)) {
-		icalcomp = icalparser_parse_string (msg->response.body);
+		icalcomp = sunone_util_icalparser_parse_string (msg->response_body->data, msg->response_body->length);
 		if (!icalcomp)
 			retval = SOUP_STATUS_MALFORMED;
 		else {
@@ -3009,21 +3057,21 @@ sunone_connection_version (SunOneConnection *connection)
 	return retval;
 }
 
-SoupStatusClass
+guint
 sunone_message_send (SunOneConnection *connection, SoupMessage *msg)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
-	SoupStatusClass status_class;
+	guint status;
 
 	setup_message (msg);
-	status_class = soup_session_send_message (priv->soup_session, msg);
-	if (status_class != SOUP_STATUS_CLASS_TRANSPORT_ERROR)
-		parse_server_response (msg, &status_class);
+	status = soup_session_send_message (priv->soup_session, msg);
+	if (!SOUP_STATUS_IS_TRANSPORT_ERROR(status))
+		parse_server_response (msg, &status);
 
-	return status_class;
+	return status;
 }
 
-SoupStatusClass
+guint
 sunone_message_send_no_parse (SunOneConnection *connection, SoupMessage *msg)
 {
 	SunOneConnectionPrivate *priv = connection->priv;
